{
        EXPLANATION:

        Hub/main/shared memory address each memory address as a byte, cog memory addresses each memory address as a group of 4 bytes or a long
        Hub memory is 32kb, cog memory is 512 longs per cog, with only 496 of them being usable by the programmer
        Cogs can write to hub memory so that cog data can be shared out to the other cogs by using a main memory address that is passed to the writing cog        
        To pass a memory address to a cog to write to, the process is as follows:
                You query the object itself with a function that sets the result := @l_variablename and provides an address
                Then you set that address to another variable name and store it
                Then you pass that variable that is storing an address, to an initialization method when starting a new object/cog
                Then you asign a new variable inside of that object, to the address value stored in that init parameter
                Then you create a corresponding pointer to store the address in pasm
                Then you readlong that pointers stored address into a pasm variable
                Then you wrlong to that pointer, which is storing the address from when you first queried it
}

{       'PST OUTPUT SHOULD LOOK LIKE THE FOLLOWING:

        Starting COG!

        Main loop [/]      Update [/]

        [var 1a]: 0                   [var 2a]:  0                  [var 3a]: 0                   [var 4a]: 0
        [var 1b]: 0                   [var 2b]:  0                  [var 3b]: 0                   [var 4b]: 0
        [var 1c]: 0                   [var 2c]:  0                  [var 3c]: 0                   [var 4c]: 0
        [var 1d]: 0                   [var 2d]:  0                  [var 3d]: 0                   [var 4d]: 0
        

        Pst1:     00000000000000100000000000000000
        Pst2:     40000000
        Pst3:     40000000
        Pst4:     00000000000000100000000000000000
        Pst5:     40000000
        Pst6:     00000000000000100000000000000000
        Pst7:     40000000
        Pst8:     00000000000000100000000000000000
        Pst9:     0
        Pst10:    0
        Pst11:    0
        Pst12:    0
        Pst13:    0
        Pst14:    0
        Pst15:    0
        Pst16:
}

con

  _CLKMODE      = XTAL1 + PLL16X                        
  _CLKFREQ      = 80_000_000
  
  BAUDRATE = 115_200

obj

  pst   : "Parallax Serial Terminal"
  
var

  long tmp 
  long  cog

  long  blinkpin  ' par                                                         ' pin used by blinker cog
  long  onticks   ' par + 4 bytes, or 1 long since cog memory uses longs only   ' pin "on" timing in system ticks 
  long  offticks  ' par + 8 bytes, or 1 long since cog memory uses longs only   ' pin "off" timing in system ticks
  long  l_pst1    ' par + 12 bytes, or 1 long since cog memory uses longs only
  long  l_pst2    ' par + 16 bytes, or 1 long since cog memory uses longs only
  long  l_pst3    ' par + 20 bytes, or 1 long since cog memory uses longs only
  long  l_pst4    ' par + 24 bytes, or 1 long since cog memory uses longs only
  long  l_pst5    ' par + 28 bytes, or 1 long since cog memory uses longs only
  long  l_pst6    ' par + 32 bytes, or 1 long since cog memory uses longs only
  long  l_pst7    ' par + 36 bytes, or 1 long since cog memory uses longs only
  long  l_pst8    ' par + 40 bytes, or 1 long since cog memory uses longs only
  long  l_pst9    ' par + 44 bytes, or 1 long since cog memory uses longs only
  long  l_pst10   ' par + 48 bytes, or 1 long since cog memory uses longs only
  long  l_pst11   ' par + 52 bytes, or 1 long since cog memory uses longs only
  long  l_pst12   ' par + 56 bytes, or 1 long since cog memory uses longs only
  long  l_pst13   ' par + 60 bytes, or 1 long since cog memory uses longs only
  long  l_pst14   ' par + 64 bytes, or 1 long since cog memory uses longs only
  long  l_pst15   ' par + 68 bytes, or 1 long since cog memory uses longs only
  long  l_pst16   ' par + 72 bytes, or 1 long since cog memory uses longs only

  

PUB Main | lpcnt1, lpcnt2
                                                                                                                       ' cog
  pst.Start( BAUDRATE )

  ' wait until any key has been pushed. This way you can switch from Propeller Tool
  ' to PST without a hurry. At least during development all my terminal-based programs
  ' use this loop.
  tmp:=-1
  repeat while tmp==-1
    pst.str( string( "== please press key to start ==",$0d ) )
    tmp:=pst.rxcheck
    if tmp==-1
       waitcnt( clkfreq+cnt )
  
  pst.Clear                     ' clears the pst screen
  pst.str( string( "Starting COG!",$0d ) )


  pstInitScreen                 ' initalizes our pst screen formatting                         

  start(17, 500, 500)           ' start(led pin#, On milliseconds, Off milliseconds)

  lpcnt1:=0                                
  lpcnt2:=0
  repeat
    charAt( 12,2 , proc[ lpcnt1 ] )
    lpcnt1 := (lpcnt1+1) & 3
    
    
      
      charAt( 28,2 , proc[ lpcnt2 ] )
      lpcnt2 := (lpcnt2+1) & 3
 
      charAt( 12, 4, " " )  ' ( "var 1a:" ) )
      pst.Dec(0)  
      pst.str(@spaces)
                                                                
      charAt( 43, 4, " " )  ' ( "var 2a:" ) )
      pst.Dec(0)      
      pst.str(@spaces)

      charAt( 72, 4, " " )  ' ( "var 3a:" ) )
      pst.Dec(0)   
      pst.str(@spaces)

      charAt( 102, 4, " " ) ' ( "var 4a:" ) )
      pst.Dec(0)   
      pst.str(@spaces)

      charAt( 12, 5, " " )  ' ( "var 1b:" ) ) 
      pst.dec(0)   
      pst.str(@spaces)

      charAt( 43, 5, " " )  ' ( "var 2b:" ) )
      pst.dec(0)
      pst.str(@spaces)

      charAt( 72, 5, " " )  ' ( "var 3b:" ) )
      pst.dec(0)
      pst.str(@spaces)

      charAt( 102, 5, " " ) ' ( "var 4b:" ) )
      pst.Dec(0)   
      pst.str(@spaces)

      charAt( 12, 6, " " )  ' ( "var 1c:" ) ) 
      pst.dec(0)   
      pst.str(@spaces)

      charAt( 43, 6, " " )  ' ( "var 2c:" ) )
      pst.dec(0)
      pst.str(@spaces)

      charAt( 72, 6, " " )  ' ( "var 3c:" ) )
      pst.dec(0)
      pst.str(@spaces)

      charAt( 102, 6, " " ) ' ( "var 4c:" ) )
      pst.Dec(0)   
      pst.str(@spaces)

      charAt( 12, 7, " " )  ' ( "var 1d:" ) ) 
      pst.dec(0)   
      pst.str(@spaces)

      charAt( 43, 7, " " )  ' ( "var 2d:" ) )
      pst.dec(0)
      pst.str(@spaces)

      charAt( 72, 7, " " )  ' ( "var 3d:" ) )
      pst.dec(0)
      pst.str(@spaces)
      
      charAt( 102, 7, " " ) ' ( "var 4d:" ) )
      pst.Dec(0)   
      pst.str(@spaces)

      charAt( 12, 21, " " )  ' ("Pst1:") )
      pst.bin(l_pst1,32)
      pst.str(@spaces)

      charAt( 12, 22, " " )  ' ("Pst2:") )
      pst.dec(l_pst2)
      pst.str(@spaces)

      charAt( 12, 23, " " )  ' ("Pst3:") )
      pst.dec(l_pst3)
      pst.str(@spaces)

      charAt( 12, 24, " " )  ' ("Pst4:") )
      pst.bin(l_pst4, 32)
      pst.str(@spaces)

      charAt( 12, 25, " " )  ' ("Pst5:") )
      pst.dec(l_pst5)
      pst.str(@spaces)
      
      charAt( 12, 26, " " )  ' ("Pst6:") )
      pst.bin(l_pst6, 32)
      pst.str(@spaces)

      charAt( 12, 27, " " )  ' ("Pst7:") )
      pst.dec(l_pst7)
      pst.str(@spaces)
      
      charAt( 12, 28, " " )  ' ("Pst8:") )
      pst.bin(l_pst8, 32)
      pst.str(@spaces)

      charAt( 12, 29, " " )  ' ("Pst9:") )
      pst.dec(l_pst9)
      pst.str(@spaces)

      charAt( 12, 30, " " )  ' ("Pst10:") )
      pst.dec(l_pst10)
      pst.str(@spaces)
      
      charAt( 12, 31, " " )  ' ("Pst11:") )
      pst.dec(l_pst11)
      pst.str(@spaces)

      charAt( 12, 32, " " )  ' ("Pst12:") )
      pst.dec(l_pst12)
      pst.str(@spaces)
      
      charAt( 12, 33, " " )  ' ("Pst13:") )
      pst.dec(l_pst13)
      pst.str(@spaces)
      
      charAt( 12, 34, " " )  ' ("Pst14:") )
      pst.dec(l_pst14)
      pst.str(@spaces)
       
      charAt( 12, 35, " " )  ' ("Pst15:") )
      pst.dec(l_pst15)
      pst.str(@spaces)

      charAt( 12, 36, " " )  ' ("Pst16:") )
      pst.dec(l_pst16)
      pst.str(@spaces)


pub start(pin, onms, offms)

'' Start blinking pin with specified on and off durations (in milliseconds)

  stop                                                          ' stop if already running

  blinkpin := pin
  onticks  := clkfreq / 1000 * onms
  offticks := clkfreq / 1000 * offms

  result := cog := cognew(@blinker, @blinkpin)  ' cognew(@address of first routine in dat code, @spin variable you decide to make "par:)         
   

PUB pstInitScreen

  ' These values are permanent values you want to display
  printAt( 1,2, string("Main loop [ ]") )
  printAt( 20,2, string("Update [ ]") )
  printAt( 3,4, string("[var 1a]:") )
  printAt( 33, 4, string( "[var 2a]:" ) )
  printAt( 63,4, string("[var 3a]:") )
  printAt( 93,4, string("[var 4a]:") )
  printAt( 3,5, string("[var 1b]:") )
  printAt( 33, 5, string( "[var 2b]:" ) )
  printAt( 63, 5, string( "[var 3b]:" ) )
  printAt( 93, 5, string( "[var 4b]:" ) )
  printAt( 3,6, string("[var 1c]:" ) )
  printAt( 33,6, string("[var 2c]:") )
  printAt( 63,6, string("[var 3c]:") )
  printAt( 93,6, string("[var 4c]:") )    
  printAt( 3,7, string("[var 1d]:") )
  printAt( 33,7, string("[var 2d]:") )
  printAt( 63,7, string("[var 3d]:") )
  printAt( 93,7, string("[var 4d]:") )

  ' The values are for troubleshooting and are temporary and can be swapped around
  printAt( 3,21, string("Pst1:") )
  printAt( 3,22, string("Pst2:") )
  printAt( 3,23, string("Pst3:") )
  printAt( 3,24, string("Pst4:") )
  printAt( 3,25, string("Pst5:") )
  printAt( 3,26, string("Pst6:") )
  printAt( 3,27, string("Pst7:") )
  printAt( 3,28, string("Pst8:") )
  printAt( 3,29, string("Pst9:") )
  printAt( 3,30, string("Pst10:") )
  printAt( 3,31, string("Pst11:") )
  printAt( 3,32, string("Pst12:") )
  printAt( 3,33, string("Pst13:") )
  printAt( 3,34, string("Pst14:") )
  printAt( 3,35, string("Pst15:") )
  printAt( 3,36, string("Pst16:") )  

dat
spaces byte "     ",0
proc   byte "-/|\"

pub charAt( x,y,c )
  ' output a character if it's in screen-range and if it's a printable character
  if x>0 and x<109 and y>0 and y<75 and c>31 and c<128
    posbuf[1]:=x
    posbuf[2]:=y
    posbuf[3]:=c
    pst.str(@posbuf)

pub printAt( x, y, text)
  ' set cursor position and print string
  posbuf[1]:=x
  posbuf[2]:=y
  posbuf[3]:=0
  pst.str(@posbuf)
  pst.str( text )
        
dat
' a buffer to assemble the bytestream for positioning
posbuf byte 2, 0, 0, 0, 0
      
  


pub stop

'' Stops blinker cog if running

  if (cog)
    cogstop(cog~ - 1)                                           ' stop cog, clear id


pub active

'' Returns non-zero (true) if blinker is active

  return cog
  


dat

                        org     0
                                  
blinker

initialize

                        add     pstPtr1, par    ' address of numeric value of bytes at bottom + address of par which in this case is set by the value of @blinkpin
                        add     pstPtr2, par
                        add     pstPtr3, par
                        add     pstPtr4, par
                        add     pstPtr5, par
                        add     pstPtr6, par
                        add     pstPtr7, par
                        add     pstPtr8, par
                        add     pstPtr9, par
                        add     pstPtr10, par
                        add     pstPtr11, par
                        add     pstPtr12, par
                        add     pstPtr13, par
                        add     pstPtr14, par
                        add     pstPtr15, par
                        add     pstPtr16, par 

                        mov     t1, par                         ' start of structure
                        rdlong  t2, t1                          ' read blink pin
                        mov     pinmask, #1                     ' create mask from pin #
                        shl     pinmask, t2
                        wrlong  pinmask, pstPtr1

                        add     t1, #4                          ' point to "on" ticks
                        rdlong  ontix, t1                       ' read on timing
                        wrlong  ontix, pstPtr2
                        
                        add     t1, #4                          ' point to "off" ticks
                        rdlong  offtix, t1                      ' read off timing
                        wrlong  offtix, pstPtr3
                        
                        mov     dira, pinmask                   ' make pin an output
                        or      outa, pinmask                   ' start on
                        wrlong  pinmask, pstPtr4
                        
                        mov     timer, ontix                    ' start with on time
                        add     timer, cnt                      ' sync with system counter

                        

blinkloop
                        add     loopCount, #1
                        waitcnt timer, offtix           ' let on time expire, load off time
                        wrlong  offtix, pstPtr5
                        andn    outa, pinmask           ' pin off, values are only 1 if both destination and source are 0, and means both have to be 1 and !1
                                                        ' bitwise and of the !source field + destination field (either has to be 0 and the other 1 for the output to be 1a)
                                                        '  0000000000000010000000000000000              0000000000000000000000000000000
                                                        ' +0000000000000000000000000000000             +0000000000000010000000000000000
                                                        ' =0000000000000010000000000000000             =0000000000000010000000000000000
                        wrlong  pinmask, pstPtr6                         
                        waitcnt timer, ontix            ' let off time expire, load on time
                        wrlong  loopCount, pstPtr9      ' pstPtr9 reads 1 when led is on                         
                        wrlong  ontix, pstPtr7                       
                        or      outa, pinmask           ' pin on, result is 1 if either value is 1, 1 or 1
                                                        ' bitwise or of source to destination (if either is 1 the output is one)
                                                        '  0000000000000010000000000000000             0000000000000010000000000000000                  0000000000000000000000000000000
                                                        ' +0000000000000010000000000000000            +0000000000000000000000000000000                 +0000000000000010000000000000000
                                                        ' =0000000000000010000000000000000            =0000000000000010000000000000000                 =0000000000000010000000000000000                        
                        wrlong  pinmask, pstPtr8
                        jmp     #blinkloop              ' loop forever
                          
' --------------------------------------------------------------------------------------------------

blinkPinPtr             long    0     ' this is a numeric number of bytes to add to the address of par when it is passed in under cognew(@firstdatroutine, @par) to the initialize portion of the dat code
onTicksPtr              long    4     ' this is a numeric number of bytes to add to the address of par when it is passed in under cognew(@firstdatroutine, @par) to the initialize portion of the dat code
offTicksPtr             long    8     ' this is a numeric number of bytes to add to the address of par when it is passed in under cognew(@firstdatroutine, @par) to the initialize portion of the dat code
pstPtr1                 long    12    ' this is a numeric number of bytes to add to the address of par when it is passed in under cognew(@firstdatroutine, @par) to the initialize portion of the dat code
pstPtr2                 long    16    ' this is a numeric number of bytes to add to the address of par when it is passed in under cognew(@firstdatroutine, @par) to the initialize portion of the dat code
pstPtr3                 long    20    ' etc
pstPtr4                 long    24
pstPtr5                 long    28
pstPtr6                 long    32
pstPtr7                 long    36
pstPtr8                 long    40
pstPtr9                 long    44
pstPtr10                long    48
pstPtr11                long    52
pstPtr12                long    56
pstPtr13                long    60
pstPtr14                long    64
pstPtr15                long    68
pstPtr16                long    72



' Standard variables here

zero                    long    0
one                     long    1
clk                     long    80000000
loopCount               long    0


' Reserved variables here 

pinmask                 res     1                               ' mask for blinker pin
ontix                   res     1                               ' on timing in system ticks
offtix                  res     1                               ' off timing in system ticks
timer                   res     1

t1                      res     1
t2                      res     1


                        fit     496
                                                        


                        
dat

{{

  This is a modified JonnyMac object, credit goes to him for the parts of this that look professional and work well

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  